import _ from 'lodash';
import * as yaml from 'yaml';
import path from 'path';
import fs from 'fs';
import * as crypto from 'crypto';
import * as minimist from 'minimist';

export const NOT_NULL = {
  isNullable: false,
};

export const NULLABLE = {
  isNullable: true,
};

export const VARCHAR = {
  type: 'varchar',
};

export const BOOL = {
  type: 'bool',
};

export const TEXT = {
  type: 'text',
};

export const SMALL_INT = {
  type: 'smallint',
};

export const INT = {
  type: 'int',
};

export const TINYINT = {
  type: 'int',
};

export const LONG_INT = {
  type: 'bigint',
};

export const TIMESTAMP = {
  type: 'timestamp',
};

export const JSON = {
  type: 'json',
};

export const ID = {
  name: 'id',
  type: 'bigint',
  isGenerated: true,
  generationStrategy: 'increment' as 'uuid' | 'increment' | 'rowid',
  isNullable: false,
  isPrimary: true,
  length: '64',
};

export const STRING_ID = {
  name: 'id',
  type: 'varchar',
  isNullable: false,
  isPrimary: true,
  length: '32',
};

export const Version = {
  name: 'version',
  type: 'int',
  comment: '版本号，事务乐观锁使用',
  default: 0,
  ...NOT_NULL,
};

export const CreatedAt = {
  name: 'created_at',
  ...LONG_INT,
  comment: '创建时间',
  default: Date.now(),
  ...NOT_NULL,
};

export const DeletedAt = {
  name: 'deleted_at',
  ...LONG_INT,
  comment: '删除时间',
  ...NULLABLE,
};

export const UpdatedAt = {
  name: 'updated_at',
  ...LONG_INT,
  comment: '修改时间',
  default: Date.now(),
  ...NOT_NULL,
};

export const UserId = {
  name: 'user_id',
  comment: '用户 ID',
  length: '32',
  ...NOT_NULL,
  ...VARCHAR,
};

export const TenantId = {
  name: 'tenant_id',
  comment: '租户 ID',
  length: '32',
  ...NOT_NULL,
  ...VARCHAR,
};

export const TABLE_DEFINITION = {};

export function chunkArray(arr: any[], len: number) {
  const chunks: any[] = [];
  let i = 0;
  const n = arr.length;

  while (i < n) {
    chunks.push(arr.slice(i, (i += len)));
  }

  return chunks;
}

export function generateRandomString(length: number = 30) {
  let result = '';
  const characters =
    'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
  const charactersLength = characters.length;
  for (let i = 0; i < length; i++) {
    result += characters.charAt(Math.floor(Math.random() * charactersLength));
  }
  return result;
}

export const generateObjectId = (
  m = Math,
  d = Date,
  h = 16,
  s = (s) => m.floor(s).toString(h),
) => s(d.now() / 1000) + ' '.repeat(h).replace(/./g, () => s(m.random() * h));

export function generateSecret() {
  const md5 = crypto.createHash('md5');
  return md5.update(generateRandomString()).digest('hex');
}
export const sleep = (minsecond: number) =>
  new Promise((resolve) => {
    setTimeout(() => {
      resolve(true);
    }, minsecond);
  });

/**
 * @Description: 初始化配置
 */
export const initConfig = () => {
  let configFilePassedInCmd = null;
  const argv = minimist(process.argv.slice(2));
  if (argv?.c) {
    configFilePassedInCmd = argv.c;
  }

  // 解析配置文件
  let rawConfigs: any[] = [];
  if (configFilePassedInCmd) {
    rawConfigs = [path.resolve(configFilePassedInCmd)];
  } else {
    rawConfigs = [
      path.resolve('/etc/skillUpp/config.yaml'),
      path.resolve('./config.yaml'),
    ];
  }

  rawConfigs = rawConfigs
    .filter(Boolean)
    .filter(fs.existsSync)
    .map((file) => fs.readFileSync(file, 'utf-8'))
    .map((content) => yaml.parse(content));

  if (rawConfigs.length === 0) {
    throw new Error('缺少配置文件：/etc/skillUpp/config.yaml');
  }

  // 递归合并
  // 优先级 env > config.{NODE_ENV}.yaml > config.yaml > //etc/skillUpp/config.yaml > 默认值
  const config = [...rawConfigs].reduce((prev, curr) => {
    return _.merge(prev, curr);
  });

  /**
   * 获得配置项的值
   * @param key 配置项的 key，可以通过 . 来选择子项，比如 app.port
   * @param defaultValue 默认值
   */
  const env = (key: string, defaultValue?: any) => {
    const envKey = key
      .split('.')
      .map((x) => x.toUpperCase())
      .join('_');
    return process.env[envKey] || _.get(config, key, defaultValue);
  };
  return {
    region: env('region', 'cn'),
    cost: {
      url: env('cost.url'),

      tenantNo: env('cost.tenantNo'),

      productNo: env('cost.productNo'),
    },
  };
};
/**
 * @Description: 生成arn
 */
export const generateArn2 = (
  resource: string,
  config: { region: string; arnName: string },
) => {
  const arn = `arn:${config.region}:${config.arnName}`;
  const index = resource.indexOf(':');
  if (index === -1) {
    // 说明 resource 只有资源类型，没有资源标识符
    return `${arn}:${resource}`;
  }
  const resourceType = resource.substring(0, index);
  const resourceId = resource.substring(index + 1);

  return `${arn}:${resourceType}:${resourceId}`;
};
