import { Injectable } from '@nestjs/common';
import { LoginDto } from './dto/req/login.dto';
import { InjectRepository } from '@nestjs/typeorm';
import { UserEntity } from '../../entities/user.entity';
import { MoreThanOrEqual, Repository } from 'typeorm';
import * as sha1 from 'sha1';
import { UnauthorizedError } from '@/common/exception/unauthorized/UnauthorizedError';
import { ActiveRecordsEntity } from '@/entities/activeRecords.entity';
import { LoginRecordDto } from './dto/loginRecord.dto';
import { JwtService } from '@nestjs/jwt';
import { plainToClass } from 'class-transformer';
import { RegisterDto } from './dto/req/register.dto';
import { LoginResDto } from './dto/res/loginRes.dto';
import { UserService } from '../user/user.service';
import { InjectRedis } from '@nestjs-modules/ioredis';
import Redis from 'ioredis';
import { MD5 } from '@/common/utils/tool';
import { axiosPostRequest } from '@/common/utils/requests/request';
import { config } from '@/config';

@Injectable()
export class AuthService {
  constructor(
    @InjectRepository(ActiveRecordsEntity)
    private readonly activeRecordsRepository: Repository<ActiveRecordsEntity>,
    private readonly jwtService: JwtService,
    private readonly userService: UserService,
    @InjectRedis() private readonly redis: Redis,
  ) {}

  // 登录
  async login(
    loginDto: LoginDto,
    loginRecord: LoginRecordDto,
  ): Promise<LoginResDto> {
    const { phone, password } = loginDto;

    // 验证用户
    const user = await this.validateUser(phone, password);

    // 更新用户登录时间
    await this.logSingIn(user);

    // 记录登录信息
    await this.loginRecords(user, loginRecord);

    // jwt
    const payload = { sub: user.id, username: user.phone };

    const token = await this.jwtService.signAsync(payload);

    // 转换并返回用户信息
    return plainToClass(
      LoginResDto,
      { ...user, token },
      { excludeExtraneousValues: true },
    );
  }

  // 注册
  async register(registerDto: RegisterDto): Promise<LoginResDto> {
    // 校验 验证码
    await this.verifyCode(registerDto.code);

    // 校验手机号是否已注册
    const exist = await this.userService.existByPhone(registerDto.phone);

    if (exist) {
      throw UnauthorizedError.default('手机号已注册');
    }

    // 创建用户
    const user = await this.userService.createUser(registerDto);

    // jwt
    const payload = { sub: user.id, username: user.phone };

    const token = await this.jwtService.signAsync(payload);

    // 转换并返回用户信息
    return plainToClass(
      LoginResDto,
      { ...user, token },
      { excludeExtraneousValues: true },
    );
  }

  // 登出
  async logout(token: string, exp: number): Promise<void> {
    // 将 token 添加到 Redis 黑名单中，并设置过期时间
    const ttl = exp - Math.floor(Date.now() / 1000);
    const key = this.getBlackListKey(token);
    await this.redis.set(key, token, 'EX', ttl);
  }

  // 获取黑名单 Key
  getBlackListKey(token: string): string {
    return `translation:blacklist:${MD5(token)}`;
  }

  // 验证 token 是否在黑名单中
  async isTokenInBlackList(token: string): Promise<boolean> {
    const key = this.getBlackListKey(token);
    const exists = await this.redis.exists(key);
    return !!exists;
  }

  // 重置密码
  async resetPwd(resetDto: RegisterDto): Promise<LoginResDto> {
    // 校验 验证码
    await this.verifyCode(resetDto.code);

    // 校验手机号是否已注册
    const exist = await this.userService.existByPhone(resetDto.phone);

    if (!exist) {
      throw UnauthorizedError.default('手机号未注册');
    }

    // 更新用户密码
    const user = await this.userService.updatePwd(resetDto);

    // jwt
    const payload = { sub: user.id, username: user.phone };

    const token = await this.jwtService.signAsync(payload);

    // 转换并返回用户信息
    return plainToClass(
      LoginResDto,
      { ...user, token },
      { excludeExtraneousValues: true },
    );
  }

  // 验证用户
  async validateUser(phone: string, password: string): Promise<UserEntity> {
    const user = await this.userService.getUserByPhone(phone);

    if (user && user.pwd === sha1(password)) {
      return user;
    }

    throw UnauthorizedError.default('账号或密码错误');
  }

  // 更新用户登录时间
  async logSingIn(user: UserEntity): Promise<void> {
    user.lastLoginTime = Math.floor(Date.now() / 1000);
    await this.userService.update(user);
  }

  // 添加登录日志
  async loginRecords(
    user: UserEntity,
    loginRecord: LoginRecordDto,
  ): Promise<void> {
    // 查询用户登录日志
    const record = await this.activeRecordsRepository.findOne({
      where: {
        userId: user.id,
        lastActiveTime: MoreThanOrEqual(new Date().setHours(0, 0, 0, 0) / 1000),
      },
      order: {
        lastActiveTime: 'DESC',
      },
    });

    loginRecord.userId = user.id;
    loginRecord.level = user.level;
    loginRecord.lastActiveTime = Math.floor(Date.now() / 1000);

    if (record) {
      // 更新记录
      await this.activeRecordsRepository.update(
        {
          id: record.id,
        },
        {
          ...loginRecord,
          times: (record.times || 0) + 1,
        },
      );
    }

    // 新增记录
    await this.activeRecordsRepository.save({
      ...loginRecord,
    });
  }

  // 校验验证码
  async verifyCode(code: number): Promise<boolean> {
    // TODO
    return true;
  }

  // 获取 asr token
  async getTxAsrToken(): Promise<any> {
    const result = await axiosPostRequest(
      config.service.tts.txAsrTokenUrl,
      {},
      {
        headers: {
          'Content-Type': 'application/json',
        },
      },
    );

    if (!result?.data) {
      throw new Error('获取ASR token失败');
    }

    return result?.data;
  }
}
