import {
  DEEPL_SOURCE_LANGUAGES,
  DEEPL_TARGET_LANGUAGES,
  FAN_YI_GOU_SOURCE_LANGUAGES,
  FAN_YI_GOU_TARGET_LANGUAGES,
} from '@/common/utils/constants';
import { Injectable } from '@nestjs/common';
import { TranslateReqDto } from './dto/req/translateReq.dto';
import { TranslateResDto } from './dto/res/translateRes.dto';
import { axiosPostRequest } from '@/common/utils/requests/request';
import { config } from '@/config';
import { BadRequestError } from '@/common/exception/badRequest/BadRequestError';
import * as crypto from 'crypto';
import * as FormData from 'form-data';
import { TranslateImageReqDto } from './dto/req/translateImageReq.dto';
import axios, { AxiosResponse } from 'axios';
import { TTSDto } from './dto/baseDto';

@Injectable()
export class TranslateService {
  constructor() {}

  // 获取翻译支持的语言
  async getLanguages(): Promise<{ source: object[]; target: object[] }> {
    return {
      source: DEEPL_SOURCE_LANGUAGES,
      target: DEEPL_TARGET_LANGUAGES,
    };
  }

  // 获取翻译狗支持的语言
  async getFanYiGouLanguages(): Promise<{
    source: object[];
    target: object[];
  }> {
    return {
      source: FAN_YI_GOU_SOURCE_LANGUAGES,
      target: FAN_YI_GOU_TARGET_LANGUAGES,
    };
  }

  // 翻译文本
  async translateText(translateDto: TranslateReqDto): Promise<TranslateResDto> {
    console.log('config.deepl', config.deepl);
    const result = await axiosPostRequest(
      `${config.deepl.endpoint}/v2/translate`,
      {
        text: [translateDto.text],
        target_lang: translateDto.target,
        source_lang: translateDto.source,
      },
      {
        headers: {
          'Content-Type': 'application/json',
          Authorization: `DeepL-Auth-Key ${config.deepl.secret}`,
        },
      },
    );

    const { translations } = result?.data;

    if (!translations?.length) {
      throw BadRequestError.default('翻译失败');
    }

    const [{ detected_source_language: source, text }] = translations;

    return { source, text };
  }

  // 获取翻译狗token
  generateToken(params) {
    params = {
      ...params,
      appid: config.service.fanYiGou.appId,
      privatekey: config.service.fanYiGou.secret,
    };

    // 按照key=value的格式，并按照参数名ASCII字典序排序
    const sortedParams = Object.keys(params)
      .sort()
      .map((key) => `${key}=${params[key]}`)
      .join('&');

    // 生成MD5哈希并转换为大写
    const token = crypto
      .createHash('md5')
      .update(sortedParams)
      .digest('hex')
      .toUpperCase();

    return token;
  }

  getFileMD5(fileBuffer: Buffer): string {
    return crypto.createHash('md5').update(fileBuffer).digest('hex');
  }

  async translateImage(file, dto: TranslateImageReqDto): Promise<any> {
    const params = {
      nonce_str: crypto.randomUUID(),
      from: dto.source,
      to: dto.target,
      md5: this.getFileMD5(file.buffer),
    };

    const token = this.generateToken(params);

    // 构建 FormData 数据
    const formData = new FormData();
    formData.append('file', file.buffer, {
      filename: file.originalname,
      contentType: file.mimetype,
    });

    formData.append('appid', config.service.fanYiGou.appId);
    formData.append('md5', params.md5);
    formData.append('nonce_str', params.nonce_str);
    formData.append('from', params.from);
    formData.append('to', params.to);
    formData.append('token', token);

    const result = await axiosPostRequest(
      `${config.service.fanYiGou.endpoint}/TranslateApi/api/image/uploadTranslateImage`,
      formData,
      {
        headers: {
          ...formData.getHeaders(),
        },
      },
    );

    if (!result?.data) {
      throw BadRequestError.default('翻译失败');
    }

    return result?.data?.data;
  }

  // 获取翻译进度
  async getProgress(taskId: string): Promise<any> {
    const params = {
      nonce_str: crypto.randomUUID(),
      tid: taskId,
    };

    const token = this.generateToken(params);

    // 构建 FormData 数据
    const formData = new FormData();
    formData.append('appid', config.service.fanYiGou.appId);
    formData.append('nonce_str', params.nonce_str);
    formData.append('tid', params.tid);
    formData.append('token', token);

    const result = await axiosPostRequest(
      `${config.service.fanYiGou.endpoint}/TranslateApi/api/image/queryImageTransProgress`,
      formData,
      {
        headers: {
          ...formData.getHeaders(),
        },
      },
    );

    if (!result?.data) {
      throw BadRequestError.default('获取翻译进度失败');
    }

    return result?.data.data;
  }

  // 下载翻译图片
  async downloadImage(taskId: string): Promise<any> {
    const params = {
      nonce_str: crypto.randomUUID(),
      tid: taskId,
    };

    const token = this.generateToken(params);

    // 构建 FormData 数据
    const formData = new FormData();
    formData.append('appid', config.service.fanYiGou.appId);
    formData.append('nonce_str', params.nonce_str);
    formData.append('tid', params.tid);
    formData.append('token', token);

    const response: AxiosResponse = await axios.post(
      `${config.service.fanYiGou.endpoint}/TranslateApi/api/image/downloadImage`,
      formData,
      {
        headers: {
          ...formData.getHeaders(),
        },
        responseType: 'stream', // 设置 responseType 为 stream
      },
    );

    if (response.status === 200 && !response?.data) {
      throw BadRequestError.default('下载翻译图片失败');
    }

    // // 设置响应头
    // res.setHeader('Content-Type', 'image/jpeg'); // 根据实际文件类型设置
    // res.setHeader('Content-Disposition', 'attachment; filename="image.jpg"'); // 设置下载文件名

    // // 将文件流返回给前端
    // response.data.pipe(res);

    // 上传图频到服务器
    const result = await this.uploadStream(response.data);

    return { url: `${config.obs.endpoint}/${result.filename}` };
  }

  async uploadStream(stream) {
    const formData = new FormData();
    formData.append('file', stream, { filename: 'image.jpg' });

    const response = await axios.post(
      `${config.chatServer.endpoint}/upload/file`,
      formData,
      {
        headers: {
          ...formData.getHeaders(),
        },
      },
    );
    if (response.status !== 200) {
      throw BadRequestError.default('上传翻译图片失败');
    }

    return response.data.data;
  }

  // tts 语音合成
  async tts(dto: TTSDto): Promise<any> {
    const result = await axiosPostRequest(
      config.service.tts.serviceUrl,
      {
        ...dto,
      },
      {
        headers: {
          'Content-Type': 'application/json',
        },
      },
    );

    if (!result?.data || !result?.data?.filename) {
      throw BadRequestError.default('语音合成失败');
    }

    return result?.data;
  }
}
